@@ -16,6 +16,8 @@
 
 from oslo_utils import uuidutils
 
+from vmware_nsxlib.v3 import nsx_constants
+
 FAKE_NAME = "fake_name"
 FAKE_SWITCH_UUID = uuidutils.generate_uuid()
 FAKE_IP_SET_UUID = uuidutils.generate_uuid()
@@ -117,15 +119,29 @@ FAKE_ROUTER = {
         "target_type": "FirewallSection",
         "target_id": FAKE_ROUTER_FW_SEC_UUID
     }],
+    "advanced_config": {
+        "external_transit_networks": ["100.64.0.0/10"],
+    },
 }
 
 FAKE_ROUTER_PORT_UUID = uuidutils.generate_uuid()
 FAKE_ROUTER_PORT = {
-    "resource_type": "LogicalRouterLinkPort",
+    "resource_type": nsx_constants.LROUTERPORT_UPLINK,
     "revision": 0,
     "id": FAKE_ROUTER_PORT_UUID,
     "display_name": FAKE_NAME,
-    "logical_router_id": FAKE_ROUTER_UUID
+    "logical_router_id": FAKE_ROUTER_UUID,
+    "subnets": [{'ip_addresses': ['172.20.1.60'], 'prefix_length': 24}]
+}
+
+FAKE_ROUTER_LINKT1_PORT_UUID = uuidutils.generate_uuid()
+FAKE_ROUTER_LINKT1_PORT = {
+    "resource_type": nsx_constants.LROUTERPORT_LINKONTIER1,
+    "revision": 0,
+    "id": FAKE_ROUTER_LINKT1_PORT_UUID,
+    "display_name": FAKE_NAME,
+    "logical_router_id": FAKE_ROUTER_UUID,
+    "linked_logical_router_port_id": {'target_id': uuidutils.generate_uuid()}
 }
 
 FAKE_QOS_PROFILE = {
@@ -331,6 +347,15 @@ FAKE_TZ = {
     "host_switch_mode": "STANDARD"
 }
 
+FAKE_TN_UUID = uuidutils.generate_uuid()
+FAKE_TN = {
+    "resource_type": "TransportNode",
+    "revision": 0,
+    "id": FAKE_TZ_UUID,
+    "display_name": FAKE_NAME,
+    "transport_zone_endpoints": [{"transport_zone_id": FAKE_TZ_UUID}]
+}
+
 FAKE_MD_UUID = uuidutils.generate_uuid()
 FAKE_URL = "http://7.7.7.70:3500/abc"
 FAKE_MD = {
@@ -487,3 +512,128 @@ FAKE_LEP = {
     "trust_ca_ids": [],
     "trust_crl_ids": [],
 }
+
+FAKE_VPN_SESS_ID = "33b2f8ce-4357-4780-8c7c-270094847395"
+FAKE_VPN_SESS = {
+    "resource_type": "PolicyBasedIPSecVPNSession",
+    "description": "con1",
+    "id": FAKE_VPN_SESS_ID,
+    "display_name": "con1",
+    "ipsec_vpn_service_id": "f5bbbd92-0c57-412f-82e6-83c73298f2e9",
+    "peer_endpoint_id": "7a821e15-93b6-46f9-9d2a-db5a164ee6e3",
+    "local_endpoint_id": "e8a3c141-b866-4cb7-91a4-e556b7bd84d6",
+    "enabled": True,
+    "policy_rules": [{
+        "id": "1211",
+        "sources": [{"subnet": "10.0.6.0/24"}],
+        "logged": False,
+        "destinations": [{"subnet": "10.0.5.0/24"}],
+        "action": "PROTECT",
+        "enabled": True,
+    }],
+}
+
+FAKE_EDGE_CLUSTER_ID = "69c6bc48-0590-4ff5-87b6-9b49e20b67e0"
+FAKE_EDGE_CLUSTER = {
+    "resource_type": "EdgeCluster",
+    "description": "edgecluster1",
+    "id": FAKE_EDGE_CLUSTER_ID,
+    "display_name": "edgecluster1",
+    "deployment_type": "VIRTUAL_MACHINE",
+    "member_node_type": "EDGE_NODE",
+    "members": [{
+        "member_index": 0,
+        "transport_node_id": "321d2746-898e-11e8-9723-000c29391f21"
+    }],
+    "cluster_profile_bindings": [{
+        "profile_id": "15d3485e-0474-4511-bd79-1506ce777baa",
+        "resource_type": "EdgeHighAvailabilityProfile"
+    }],
+}
+
+FAKE_TIERO_ROUTER_ID = "67927d95-18d3-4763-9eb1-a45ff0e63bbe"
+FAKE_TIERO_ROUTER = {
+    "resource_type": "LogicalRouter",
+    "description": "Provider Logical Router(Tier0)",
+    "id": FAKE_TIERO_ROUTER_ID,
+    "display_name": "PLR-1 LogicalRouterTier0",
+    "edge_cluster_id": FAKE_EDGE_CLUSTER_ID,
+    "firewall_sections": [{
+        "is_valid": True,
+        "target_type": "FirewallSection",
+        "target_id": "c3d80576-e340-403d-a2d0-f4a72a1db6e3"
+    }],
+    "advanced_config": {
+        "external_transit_networks": ["100.64.0.0/16"],
+        "internal_transit_network": "169.254.0.0/28"
+    },
+    "router_type": "TIER0",
+    "high_availability_mode": "ACTIVE_STANDBY",
+    "failover_mode": "NON_PREEMPTIVE",
+}
+
+
+FAKE_TRANS_NODE_ID = "f5a2b5ca-8dba-11e8-9799-020039422cc8"
+FAKE_TRANS_NODE = {
+    "resource_type": "TransportNode",
+    "id": FAKE_TRANS_NODE_ID,
+    "display_name": FAKE_TRANS_NODE_ID,
+    "maintenance_mode": "DISABLED",
+    "transport_zone_endpoints": [{
+        "transport_zone_id": FAKE_TZ_UUID,
+        "transport_zone_profile_ids": [{
+            "profile_id": "52035bb3-ab02-4a08-9884-18631312e50a",
+            "resource_type": "BfdHealthMonitoringProfile"
+        }]
+    }],
+    "node_id": "f5a2b5ca-8dba-11e8-9799-020039422cc8"
+}
+
+FAKE_MANAGER_IP1 = "10.192.210.181"
+FAKE_MANAGER_IP2 = "10.192.210.182"
+FAKE_CLUSTER_NODES_CONFIG = [{
+    "resource_type": "ClusterNodeConfig",
+    "manager_role": {
+        "type": "ManagementClusterRoleConfig",
+        "mgmt_cluster_listen_addr": {
+            "port": 0,
+            "ip_address": FAKE_MANAGER_IP1
+        },
+        "api_listen_addr": {
+            "port": 443,
+            "ip_address": FAKE_MANAGER_IP1
+        },
+        "mgmt_plane_listen_addr": {
+            "port": 5671,
+            "ip_address": FAKE_MANAGER_IP1
+        }
+    },
+    "appliance_mgmt_listen_addr": FAKE_MANAGER_IP1
+}, {
+    "resource_type": "ClusterNodeConfig",
+    "controller_role": {
+        "type": "ControllerClusterRoleConfig",
+        "control_cluster_listen_addr": {
+            "port": 7777,
+            "ip_address": "127.0.0.1"
+        },
+    },
+}, {
+    "resource_type": "ClusterNodeConfig",
+    "manager_role": {
+        "type": "ManagementClusterRoleConfig",
+        "mgmt_cluster_listen_addr": {
+            "port": 0,
+            "ip_address": FAKE_MANAGER_IP2
+        },
+        "api_listen_addr": {
+            "port": 443,
+            "ip_address": FAKE_MANAGER_IP2
+        },
+        "mgmt_plane_listen_addr": {
+            "port": 5671,
+            "ip_address": FAKE_MANAGER_IP2
+        }
+    },
+    "appliance_mgmt_listen_addr": FAKE_MANAGER_IP2
+}]
