@@ -427,6 +427,56 @@ class LogicalPortTestCase(BaseTestResource):
 
         self.assertEqual(fake_port, result)
 
+    def test_create_logical_port_with_tn_uuid(self):
+        """Test creating port with transport_node_uuid."""
+        fake_port = copy.deepcopy(test_constants.FAKE_CONTAINER_PORT)
+        fake_port['parent_vif_id'] = None
+        fake_port_ctx = fake_port['attachment']['context']
+        fake_port_ctx['vif_type'] = 'INDEPENDENT'
+        fake_port_ctx['transport_node_uuid'] = test_constants.FAKE_TN_UUID
+
+        profile_dicts = self._get_profile_dicts(fake_port)
+        pkt_classifiers, binding_repr = self._get_pktcls_bindings()
+        fake_port['address_bindings'] = binding_repr
+
+        mocked_resource = self.get_mocked_resource()
+        switch_profile = resources.SwitchingProfile
+
+        mocked_resource.create(
+            fake_port['logical_switch_id'],
+            fake_port['attachment']['id'],
+            traffic_tag=fake_port_ctx['vlan_tag'],
+            address_bindings=pkt_classifiers,
+            switch_profile_ids=switch_profile.build_switch_profile_ids(
+                mock.Mock(), *profile_dicts),
+            vif_type=fake_port_ctx['vif_type'], app_id=fake_port_ctx['app_id'],
+            allocate_addresses=fake_port_ctx['allocate_addresses'],
+            tn_uuid=fake_port_ctx['transport_node_uuid'])
+
+        resp_body = {
+            'logical_switch_id': fake_port['logical_switch_id'],
+            'switching_profile_ids': fake_port['switching_profile_ids'],
+            'attachment': {
+                'attachment_type': 'VIF',
+                'id': fake_port['attachment']['id'],
+                'context': {
+                    'resource_type': 'VifAttachmentContext',
+                    'allocate_addresses': 'Both',
+                    'app_id': fake_port_ctx['app_id'],
+                    'vif_type': 'INDEPENDENT',
+                    'transport_node_uuid': test_constants.FAKE_TN_UUID,
+                }
+            },
+            'admin_state': 'UP',
+            'address_bindings': fake_port['address_bindings']
+        }
+
+        test_client.assert_json_call(
+            'post', mocked_resource,
+            'https://1.2.3.4/api/v1/logical-ports',
+            data=jsonutils.dumps(resp_body, sort_keys=True),
+            headers=self.default_headers())
+
     def test_delete_resource(self):
         """Test deleting port."""
         super(LogicalPortTestCase, self).test_delete_resource(
@@ -589,14 +639,24 @@ class LogicalRouterTestCase(BaseTestResource):
         router = self.get_mocked_resource()
         tier0_router = True
         description = 'dummy'
+        tz_id = 'tz_id'
+        allocation_pool = {
+            'allocation_pool_type': 'LoadBalancerAllocationPool',
+            'allocation_size': 'SMALL'
+        }
         router.create(fake_router['display_name'], None, None, tier0_router,
-                      description=description)
+                      description=description, transport_zone_id=tz_id,
+                      allocation_pool=allocation_pool)
 
         data = {
             'display_name': fake_router['display_name'],
             'router_type': 'TIER0' if tier0_router else 'TIER1',
             'tags': None,
-            'description': description
+            'description': description,
+            'advanced_config': {'transport_zone_id': tz_id},
+            'allocation_profile': {
+                'allocation_pool': allocation_pool
+            }
         }
 
         test_client.assert_json_call(
@@ -605,6 +665,33 @@ class LogicalRouterTestCase(BaseTestResource):
             data=jsonutils.dumps(data, sort_keys=True),
             headers=self.default_headers())
 
+    def test_update_logical_router(self):
+        fake_router = test_constants.FAKE_ROUTER.copy()
+        router = self.get_mocked_resource()
+        uuid = fake_router['id']
+
+        name = 'dummy'
+        description = 'dummy'
+        edge_cluster_id = 'ec_id'
+        tz_id = 'tz_id'
+        with mock.patch.object(router.client, 'get',
+                               return_value=fake_router),\
+            mock.patch("vmware_nsxlib.v3.NsxLib.get_version",
+                       return_value='2.2.0'):
+            router.update(uuid, display_name=name, description=description,
+                          edge_cluster_id=edge_cluster_id,
+                          transport_zone_id=tz_id)
+
+        fake_router["display_name"] = name
+        fake_router["description"] = description
+        fake_router["edge_cluster_id"] = edge_cluster_id
+        fake_router["advanced_config"]['transport_zone_id'] = tz_id
+        test_client.assert_json_call(
+            'put', router,
+            'https://1.2.3.4/api/v1/logical-routers/%s' % uuid,
+            data=jsonutils.dumps(fake_router, sort_keys=True),
+            headers=self.default_headers())
+
     def test_force_delete_logical_router(self):
         """Test force deleting router"""
         router = self.get_mocked_resource()
@@ -631,14 +718,16 @@ class LogicalRouterTestCase(BaseTestResource):
             test_constants.FAKE_ROUTER_UUID, router_body=fake_router)
         self.assertEqual(test_constants.FAKE_ROUTER_FW_SEC_UUID, section_id)
 
-    def _test_nat_rule_create(self, nsx_version, add_bypas_arg):
+    def _test_nat_rule_create(self, nsx_version, add_bypas_arg=True,
+                              action='SNAT', expect_failure=False):
         router = self.get_mocked_resource()
-        action = 'SNAT'
         translated_net = '1.1.1.1'
         priority = 10
+        display_name = 'fake_name'
 
         data = {
             'action': action,
+            'display_name': display_name,
             'enabled': True,
             'translated_network': translated_net,
             'rule_priority': priority
@@ -650,11 +739,19 @@ class LogicalRouterTestCase(BaseTestResource):
         # Ignoring 'bypass_firewall' with version 1.1
         with mock.patch("vmware_nsxlib.v3.NsxLib.get_version",
                         return_value=nsx_version):
-            router.add_nat_rule(test_constants.FAKE_ROUTER_UUID,
-                                action=action,
-                                translated_network=translated_net,
-                                rule_priority=priority,
-                                bypass_firewall=False)
+            try:
+                router.add_nat_rule(test_constants.FAKE_ROUTER_UUID,
+                                    action=action,
+                                    translated_network=translated_net,
+                                    rule_priority=priority,
+                                    bypass_firewall=False,
+                                    display_name=display_name)
+            except exceptions.InvalidInput as e:
+                if expect_failure:
+                    return
+                else:
+                    self.fail("Failed to create NAT rule: %s", e)
+
             test_client.assert_json_call(
                 'post', router,
                 ('https://1.2.3.4/api/v1/logical-routers/%s/nat/rules' %
@@ -664,11 +761,25 @@ class LogicalRouterTestCase(BaseTestResource):
 
     def test_nat_rule_create_v1(self):
         # Ignoring 'bypass_firewall' with version 1.1
-        self._test_nat_rule_create('1.1.0', False)
+        self._test_nat_rule_create('1.1.0', add_bypas_arg=False)
 
     def test_nat_rule_create_v2(self):
         # Sending 'bypass_firewall' with version 1.1
-        self._test_nat_rule_create('2.0.0', True)
+        self._test_nat_rule_create('2.0.0')
+
+    def test_nat_rule_create_v22_NO_DNAT(self):
+        # NO_DNAT is supported from 2.2 & up
+        self._test_nat_rule_create('2.2.0', action='NO_DNAT')
+
+    def test_nat_rule_create_v2_NO_DNAT(self):
+        # NO_DNAT is supported from 2.2 & up
+        self._test_nat_rule_create('2.0.0', action='NO_DNAT',
+                                   expect_failure=True)
+
+    def test_nat_rule_create_invalid(self):
+        # NO_DNAT is supported from 2.2 & up
+        self._test_nat_rule_create('2.0.0', action='INVALID',
+                                   expect_failure=True)
 
     def test_nat_rule_list(self):
         router = self.get_mocked_resource()
@@ -814,6 +925,46 @@ class LogicalRouterTestCase(BaseTestResource):
              'advertisement/rules' % router_id),
             headers=self.default_headers())
 
+    def test_get_debug_info(self):
+        router = self.get_mocked_resource()
+        router_id = test_constants.FAKE_ROUTER_UUID
+        router.get_debug_info(router_id)
+        test_client.assert_json_call(
+            'get', router,
+            ('https://1.2.3.4/api/v1/logical-routers/%s/'
+             'debug-info?format=text' % router_id),
+            headers=self.default_headers())
+
+    def test_get_transportzone_id_empty(self):
+        # Tier0 router may fail to provide TZ id if it
+        # is not yet connected with any Tier1 router
+        router = self.get_mocked_resource()
+        router_id = test_constants.FAKE_ROUTER_UUID
+        faked_responds = {
+            'componentInfo': [{
+                'componentType': nsx_constants.ROUTER_TYPE_TIER0_DR,
+                'transportZoneId': None
+            }]
+        }
+        with mock.patch.object(router.client, 'get',
+                               return_value=faked_responds):
+            res = router.get_transportzone_id(router_id)
+            self.assertIsNone(res)
+
+    def test_get_transportzone_id(self):
+        router = self.get_mocked_resource()
+        router_id = test_constants.FAKE_ROUTER_UUID
+        faked_responds = {
+            'componentInfo': [{
+                'componentType': nsx_constants.ROUTER_TYPE_TIER0_DR,
+                'transportZoneId': ['faked_id']
+            }]
+        }
+        with mock.patch.object(router.client, 'get',
+                               return_value=faked_responds):
+            res = router.get_transportzone_id(router_id)
+            self.assertEqual('faked_id', res)
+
 
 class LogicalRouterPortTestCase(BaseTestResource):
 
@@ -837,7 +988,8 @@ class LogicalRouterPortTestCase(BaseTestResource):
             'tags': [],
             'service_bindings': [{'service_id': {
                 'target_type': 'LogicalService',
-                'target_id': fake_relay_uuid}}]
+                'target_id': fake_relay_uuid}}],
+            'linked_logical_switch_port_id': {'target_id': None}
         }
 
         with mock.patch("vmware_nsxlib.v3.NsxLib.get_version",
@@ -872,21 +1024,14 @@ class LogicalRouterPortTestCase(BaseTestResource):
             mock.patch("vmware_nsxlib.v3.NsxLib.get_version",
                        return_value='2.0.0'):
             lrport.update(uuid, relay_service_uuid=fake_relay_uuid)
-            data = {
-                'id': uuid,
-                'display_name': fake_router_port['display_name'],
-                'logical_router_id': fake_router_port['logical_router_id'],
-                'resource_type': fake_router_port['resource_type'],
-                "revision": 0,
-                'service_bindings': [{'service_id': {
-                    'target_type': 'LogicalService',
-                    'target_id': fake_relay_uuid}}]
-            }
+            fake_router_port['service_bindings'] = [{'service_id': {
+                'target_type': 'LogicalService',
+                'target_id': fake_relay_uuid}}]
 
             test_client.assert_json_call(
                 'put', lrport,
                 'https://1.2.3.4/api/v1/logical-router-ports/%s' % uuid,
-                data=jsonutils.dumps(data, sort_keys=True),
+                data=jsonutils.dumps(fake_router_port, sort_keys=True),
                 headers=self.default_headers())
 
     def test_get_logical_router_port_by_router_id(self):
@@ -921,6 +1066,66 @@ class LogicalRouterPortTestCase(BaseTestResource):
             'logical_switch_id=%s' % switch_id,
             headers=self.default_headers())
 
+    def test_get_tier1_link_port(self):
+        """Test getting a Tier0 router uplink port by router id."""
+        router_id = test_constants.FAKE_ROUTER_PORT['logical_router_id']
+
+        # No ports found - raise an exception
+        lrport = self.get_mocked_resource(response={'results': []})
+        self.assertRaises(exceptions.ResourceNotFound,
+                          lrport.get_tier1_link_port,
+                          router_id)
+
+        # Non uplink ports found - raise an exception
+        lrport = self.get_mocked_resource(response={'results': [
+            test_constants.FAKE_ROUTER_PORT]})
+        self.assertRaises(exceptions.ResourceNotFound,
+                          lrport.get_tier1_link_port,
+                          router_id)
+
+        # uplink port exists
+        lrport = self.get_mocked_resource(response={'results': [
+            test_constants.FAKE_ROUTER_LINKT1_PORT]})
+        result = lrport.get_tier1_link_port(router_id)
+        self.assertEqual(test_constants.FAKE_ROUTER_LINKT1_PORT, result)
+
+    def test_get_tier0_uplink_port(self):
+        """Test getting a Tier0 router uplink port by router id."""
+        router_id = test_constants.FAKE_ROUTER_PORT['logical_router_id']
+
+        # No ports found - return None
+        lrport = self.get_mocked_resource(response={'results': []})
+        result = lrport.get_tier0_uplink_port(router_id)
+        self.assertIsNone(result)
+
+        # Non uplink ports found - return None
+        lrport = self.get_mocked_resource(response={'results': [
+            test_constants.FAKE_ROUTER_LINKT1_PORT]})
+        result = lrport.get_tier0_uplink_port(router_id)
+        self.assertIsNone(result)
+
+        # uplink port exists
+        lrport = self.get_mocked_resource(response={'results': [
+            test_constants.FAKE_ROUTER_PORT]})
+        result = lrport.get_tier0_uplink_port(router_id)
+        self.assertEqual(test_constants.FAKE_ROUTER_PORT, result)
+
+    def test_get_tier0_uplink_port_ips(self):
+        """Test getting a Tier0 router uplink port by router id."""
+        router_id = test_constants.FAKE_ROUTER_PORT['logical_router_id']
+
+        # No ports found - return empty list
+        lrport = self.get_mocked_resource(response={'results': []})
+        result = lrport.get_tier0_uplink_ips(router_id)
+        self.assertEqual(0, len(result))
+
+        # uplink port exists, return ips
+        lrport = self.get_mocked_resource(response={'results': [
+            test_constants.FAKE_ROUTER_PORT]})
+        result = lrport.get_tier0_uplink_ips(router_id)
+        self.assertEqual(1, len(result))
+        self.assertEqual('172.20.1.60', result[0])
+
 
 class IpPoolTestCase(BaseTestResource):
 
@@ -1042,6 +1247,26 @@ class IpPoolTestCase(BaseTestResource):
         else:
             self.fail("shouldn't happen")
 
+    def test_delete_ip_pool(self):
+        fake_ip_pool = test_constants.FAKE_IP_POOL.copy()
+        pool = self.get_mocked_resource(response=fake_ip_pool)
+        uuid = fake_ip_pool['id']
+        pool.delete(uuid)
+        test_client.assert_json_call(
+            'delete', pool,
+            'https://1.2.3.4/api/v1/pools/ip-pools/%s' % uuid,
+            headers=self.default_headers())
+
+    def test_force_delete_ip_pool(self):
+        fake_ip_pool = test_constants.FAKE_IP_POOL.copy()
+        pool = self.get_mocked_resource(response=fake_ip_pool)
+        uuid = fake_ip_pool['id']
+        pool.delete(uuid, force=True)
+        test_client.assert_json_call(
+            'delete', pool,
+            'https://1.2.3.4/api/v1/pools/ip-pools/%s?force=True' % uuid,
+            headers=self.default_headers())
+
     def test_update_ip_pool_name(self):
         fake_ip_pool = test_constants.FAKE_IP_POOL.copy()
         pool = self.get_mocked_resource(response=fake_ip_pool)
@@ -1277,6 +1502,19 @@ class TransportZone(BaseTestResource):
             self.assertEqual(tz.HOST_SWITCH_MODE_STANDARD, tz_mode)
 
 
+class TransportNode(BaseTestResource):
+
+    def setUp(self):
+        super(TransportNode, self).setUp(core_resources.NsxLibTransportNode)
+
+    def test_get_transport_zones(self):
+        fake_tn = test_constants.FAKE_TN.copy()
+        tn = self.get_mocked_resource()
+        with mock.patch.object(tn.client, 'url_get', return_value=fake_tn):
+            tzs = tn.get_transport_zones(fake_tn['id'])
+            self.assertEqual([test_constants.FAKE_TZ_UUID], tzs)
+
+
 class MetadataProxy(BaseTestResource):
 
     def setUp(self):
@@ -1300,6 +1538,18 @@ class MetadataProxy(BaseTestResource):
                 data=jsonutils.dumps(fake_md, sort_keys=True),
                 headers=self.default_headers())
 
+    def test_get_md_proxy_status(self):
+        """Test getting proxy_status."""
+        mocked_resource = self.get_mocked_resource()
+        attachment_id = 'd84ba3b8-9201-4591-8264-aad289e762ee'
+        logical_switch_id = 'e11803a2-2d3e-452b-a834-aeb94940d272'
+        mocked_resource.get_md_proxy_status(attachment_id, logical_switch_id)
+        test_client.assert_json_call(
+            'get', mocked_resource,
+            "https://1.2.3.4/api/v1/md-proxies/%s/%s/status" %
+            (attachment_id, logical_switch_id),
+            headers=self.default_headers())
+
 
 class NsxLibSwitchTestCase(BaseTestResource):
 
@@ -1308,12 +1558,13 @@ class NsxLibSwitchTestCase(BaseTestResource):
             core_resources.NsxLibLogicalSwitch)
         self._tz_id = uuidutils.generate_uuid()
 
-    def _create_body(self, admin_state=nsx_constants.ADMIN_STATE_UP,
+    def _create_body(self, display_name="fake_name",
+                     admin_state=nsx_constants.ADMIN_STATE_UP,
                      vlan_id=None, description=None, trunk_vlan=None):
         body = {
             "transport_zone_id": self._tz_id,
             "replication_mode": "MTEP",
-            "display_name": "fake_name",
+            "display_name": display_name,
             "tags": [],
             "admin_state": admin_state
         }
@@ -1416,6 +1667,17 @@ class NsxLibSwitchTestCase(BaseTestResource):
                               mocks.FAKE_NAME, self._tz_id, [],
                               trunk_vlan_range=trunk_vlan)
 
+    def test_create_logical_switch_illegal_name(self):
+        """Test creating switch with illegal name that will be escaped"""
+        ls = self.get_mocked_resource()
+        ls.create(mocks.FAKE_NAME + ';|=,~@', self._tz_id, [])
+        data = self._create_body(display_name=mocks.FAKE_NAME + '......')
+        test_client.assert_json_call(
+            'post', ls,
+            'https://1.2.3.4/api/v1/logical-switches',
+            data=jsonutils.dumps(data, sort_keys=True),
+            headers=self.default_headers())
+
     def test_delete_resource(self):
         """Test deleting switch"""
         super(NsxLibSwitchTestCase, self).test_delete_resource(
@@ -1724,6 +1986,24 @@ class NodeHttpServicePropertiesTestCase(BaseTestResource):
                 headers=self.default_headers())
 
 
+class TestNsxlibClusterNodesConfigTestCase(BaseTestResource):
+    def setUp(self):
+        super(TestNsxlibClusterNodesConfigTestCase, self).setUp(
+            resources.NsxlibClusterNodesConfig)
+
+    def test_delete_resource(self):
+        self.skipTest("The action is not supported by this resource")
+
+    def test_get_managers_ips(self):
+        mocked_resource = self.get_mocked_resource()
+        body = {'results': test_constants.FAKE_CLUSTER_NODES_CONFIG}
+        with mock.patch.object(mocked_resource.client, "url_get",
+                               return_value=body):
+            result = mocked_resource.get_managers_ips()
+            self.assertEqual([test_constants.FAKE_MANAGER_IP1,
+                              test_constants.FAKE_MANAGER_IP2], result)
+
+
 class DummyCachedResource(utils.NsxLibApiBase):
 
     @property
