@@ -134,7 +134,14 @@ class NsxLibBase(object):
             url += "&cursor=%d" % cursor
         if page_size:
             url += "&page_size=%d" % page_size
-        return self.client.url_get(url)
+
+        # Retry the search on case of error
+        @utils.retry_upon_exception(exceptions.NsxIndexingInProgress,
+                                    max_attempts=self.client.max_attempts)
+        def do_search(url):
+            return self.client.url_get(url)
+
+        return do_search(url)
 
     def search_all_by_tags(self, tags, resource_type=None):
         """Return all the results searched based on tags."""
@@ -244,6 +251,8 @@ class NsxLib(NsxLibBase):
             self.client, self.nsxlib_config, nsxlib=self)
         self.transport_zone = core_resources.NsxLibTransportZone(
             self.client, self.nsxlib_config, nsxlib=self)
+        self.transport_node = core_resources.NsxLibTransportNode(
+            self.client, self.nsxlib_config, nsxlib=self)
         self.relay_service = core_resources.NsxLibDhcpRelayService(
             self.client, self.nsxlib_config, nsxlib=self)
         self.relay_profile = core_resources.NsxLibDhcpRelayProfile(
@@ -287,6 +296,8 @@ class NsxLib(NsxLibBase):
             self.client, self.nsxlib_config, nsxlib=self)
         self.http_services = resources.NodeHttpServiceProperties(
             self.client, self.nsxlib_config, nsxlib=self)
+        self.cluster_nodes = resources.NsxlibClusterNodesConfig(
+            self.client, self.nsxlib_config, nsxlib=self)
 
         # Update tag limits
         self.tag_limits = self.get_tag_limits()
@@ -304,7 +315,25 @@ class NsxLib(NsxLibBase):
         self.nsx_version = node.get('node_version')
         return self.nsx_version
 
+    def export_restricted(self):
+        node = self.client.get("node")
+        return node.get('export_type') is 'RESTRICTED'
+
     def feature_supported(self, feature):
+        if (version.LooseVersion(self.get_version()) >=
+                version.LooseVersion(nsx_constants.NSX_VERSION_2_4_0)):
+            # Features available since 2.4
+            if (feature == nsx_constants.FEATURE_ENS_WITH_SEC):
+                return True
+            if (feature == nsx_constants.FEATURE_ICMP_STRICT):
+                return True
+
+        if (version.LooseVersion(self.get_version()) >=
+                version.LooseVersion(nsx_constants.NSX_VERSION_2_3_0)):
+            # Features available since 2.3
+            if (feature == nsx_constants.FEATURE_ROUTER_ALLOCATION_PROFILE):
+                return True
+
         if (version.LooseVersion(self.get_version()) >=
             version.LooseVersion(nsx_constants.NSX_VERSION_2_2_0)):
             # Features available since 2.2
@@ -312,7 +341,9 @@ class NsxLib(NsxLibBase):
                 feature == nsx_constants.FEATURE_IPSEC_VPN or
                 feature == nsx_constants.FEATURE_ON_BEHALF_OF or
                 feature == nsx_constants.FEATURE_RATE_LIMIT or
-                feature == nsx_constants.FEATURE_TRUNK_VLAN):
+                feature == nsx_constants.FEATURE_TRUNK_VLAN or
+                feature == nsx_constants.FEATURE_ROUTER_TRANSPORT_ZONE or
+                feature == nsx_constants.FEATURE_NO_DNAT_NO_SNAT):
                 return True
 
         if (version.LooseVersion(self.get_version()) >=
@@ -352,7 +383,15 @@ class NsxPolicyLib(NsxLibBase):
         self.service = policy_resources.NsxPolicyL4ServiceApi(self.policy_api)
         self.icmp_service = policy_resources.NsxPolicyIcmpServiceApi(
             self.policy_api)
-        self.comm_profile = policy_resources.NsxPolicyCommunicationProfileApi(
+        self.ip_protocol_service = (
+            policy_resources.NsxPolicyIPProtocolServiceApi(
+                self.policy_api))
+        self.tier0 = policy_resources.NsxPolicyTier0Api(self.policy_api)
+        self.tier1 = policy_resources.NsxPolicyTier1Api(self.policy_api)
+        self.tier1_segment = policy_resources.NsxPolicyTier1SegmentApi(
+            self.policy_api)
+        self.segment = policy_resources.NsxPolicySegmentApi(self.policy_api)
+        self.segment_port = policy_resources.NsxPolicySegmentPortApi(
             self.policy_api)
         self.comm_map = policy_resources.NsxPolicyCommunicationMapApi(
             self.policy_api)
@@ -365,7 +404,33 @@ class NsxPolicyLib(NsxLibBase):
     def keepalive_section(self):
         return 'infra'
 
+    def get_version(self):
+        """Get the NSX Policy manager version
+
+        Currently the backend does not support it, so the nsx-manager api
+        will be used temporarily.
+        """
+        if self.nsx_version:
+            return self.nsx_version
+
+        manager_client = client.NSX3Client(
+            self.cluster,
+            nsx_api_managers=self.nsxlib_config.nsx_api_managers,
+            max_attempts=self.nsxlib_config.max_attempts,
+            url_path_base=client.NSX3Client.NSX_V1_API_PREFIX,
+            rate_limit_retry=self.nsxlib_config.rate_limit_retry)
+
+        node = manager_client.get('node')
+        self.nsx_version = node.get('node_version')
+        return self.nsx_version
+
     def feature_supported(self, feature):
+        if (version.LooseVersion(self.get_version()) >=
+                version.LooseVersion(nsx_constants.NSX_VERSION_2_4_0)):
+            # Features available since 2.4
+            if (feature == nsx_constants.FEATURE_NSX_POLICY_NETWORKING):
+                return True
+
         return (feature == nsx_constants.FEATURE_NSX_POLICY)
 
     @property
