@@ -34,13 +34,17 @@ def http_error_to_exception(status_code, error_code):
         requests.codes.NOT_FOUND:
             {'202': exceptions.BackendResourceNotFound,
              'default': exceptions.ResourceNotFound},
+        requests.codes.BAD_REQUEST:
+            {'60508': exceptions.NsxIndexingInProgress,
+             '500045': exceptions.NsxPendingDelete},
         requests.codes.CONFLICT: exceptions.StaleRevision,
         requests.codes.PRECONDITION_FAILED: exceptions.StaleRevision,
         requests.codes.INTERNAL_SERVER_ERROR:
             {'99': exceptions.ClientCertificateNotTrusted},
         requests.codes.FORBIDDEN:
             {'98': exceptions.BadXSRFToken},
-        requests.codes.TOO_MANY_REQUESTS: exceptions.TooManyRequests}
+        requests.codes.TOO_MANY_REQUESTS: exceptions.TooManyRequests,
+        requests.codes.SERVICE_UNAVAILABLE: exceptions.ServiceUnavailable}
 
     if status_code in errors:
         if isinstance(errors[status_code], dict):
@@ -62,6 +66,7 @@ class RESTClient(object):
         'get': [requests.codes.ok],
         'post': [requests.codes.created, requests.codes.ok],
         'put': [requests.codes.created, requests.codes.ok],
+        'patch': [requests.codes.created, requests.codes.ok],
         'delete': [requests.codes.ok]
     }
 
@@ -100,6 +105,9 @@ class RESTClient(object):
         return self.url_post(resource, body, headers=headers,
                              expected_results=expected_results)
 
+    def patch(self, resource='', body=None, headers=None):
+        return self.url_patch(resource, body, headers=headers)
+
     def url_list(self, url, headers=None, silent=False):
         concatenate_response = self.url_get(url, headers=headers)
         cursor = concatenate_response.get('cursor', NULL_CURSOR_PREFIX)
@@ -128,6 +136,9 @@ class RESTClient(object):
         return self._rest_call(url, method='POST', body=body, headers=headers,
                                expected_results=expected_results)
 
+    def url_patch(self, url, body, headers=None):
+        return self._rest_call(url, method='PATCH', body=body, headers=headers)
+
     def _raise_error(self, status_code, operation, result_msg,
                      error_code=None):
         error = http_error_to_exception(status_code, error_code)
@@ -296,11 +307,12 @@ class NSX3Client(JSONRESTClient):
     def _rest_call(self, url, **kwargs):
         if self.rate_limit_retry:
             # If too many requests are handled by the nsx at the same time,
-            # error "429: Too Many Requests" will be returned.
+            # error "429: Too Many Requests" or "503: Server Unavailable"
+            # will be returned.
             # the client is expected to retry after a random 400-600 milli,
             # and later exponentially until 5 seconds wait
             @utils.retry_random_upon_exception(
-                exceptions.TooManyRequests,
+                exceptions.ServerBusy,
                 max_attempts=self.max_attempts)
             def _rest_call_with_retry(self, url, **kwargs):
                 return super(NSX3Client, self)._rest_call(url, **kwargs)
