@@ -33,10 +33,12 @@ from requests import adapters
 from requests import exceptions as requests_exceptions
 import six
 import six.moves.urllib.parse as urlparse
+import tenacity
 
 from vmware_nsxlib._i18n import _
 from vmware_nsxlib.v3 import client as nsx_client
 from vmware_nsxlib.v3 import exceptions
+from vmware_nsxlib.v3 import utils
 
 
 LOG = log.getLogger(__name__)
@@ -108,7 +110,7 @@ class TimeoutSession(requests.Session):
         def request_with_retry_on_ssl_error(*args, **kwargs):
             try:
                 return super(TimeoutSession, self).request(*args, **kwargs)
-            except OpenSSL.SSL.Error:
+            except (IOError, OpenSSL.SSL.Error):
                 # This can happen when connection tries to access certificate
                 # file it was opened with (renegotiation?)
                 # Proper way to solve this would be to pass in-memory cert
@@ -221,6 +223,9 @@ class NSXRequestsHTTPProvider(AbstractHTTPProvider):
     def is_connection_exception(self, exception):
         return isinstance(exception, requests_exceptions.ConnectionError)
 
+    def is_conn_open_exception(self, exception):
+        return isinstance(exception, requests_exceptions.ConnectTimeout)
+
     def get_default_headers(self, session, provider, allow_overwrite_header):
         """Get the default headers that should be added to future requests"""
         session.default_headers = {}
@@ -373,8 +378,8 @@ class ClusteredAPI(object):
 
     def __init__(self, providers,
                  http_provider,
-                 min_conns_per_pool=1,
-                 max_conns_per_pool=500,
+                 min_conns_per_pool=0,
+                 max_conns_per_pool=20,
                  keepalive_interval=33):
 
         self._http_provider = http_provider
@@ -397,7 +402,6 @@ class ClusteredAPI(object):
 
         def _create_conn(p):
             def _conn():
-                # called when a pool needs to create a new connection
                 return self._http_provider.new_connection(self, p)
 
             return _conn
@@ -425,8 +429,8 @@ class ClusteredAPI(object):
             conns.spawn(self._validate, endpoint)
         eventlet.sleep(0)
         while conns.running():
-            if (self.health == ClusterHealth.GREEN
-                    or self.health == ClusterHealth.ORANGE):
+            if (self.health == ClusterHealth.GREEN or
+                self.health == ClusterHealth.ORANGE):
                 # only wait for 1 or more endpoints to reduce init time
                 break
             eventlet.sleep(0.5)
@@ -502,13 +506,39 @@ class ClusteredAPI(object):
                         {'ep': endpoint, 'err': e})
 
     def _select_endpoint(self):
-        # check for UP state until exhausting all endpoints
-        seen, total = 0, len(self._endpoints.values())
-        while seen < total:
-            endpoint = next(self._endpoint_schedule)
-            if endpoint.state == EndpointState.UP:
-                return endpoint
-            seen += 1
+        """Return an endpoint in UP state.
+
+        Go over all endpoint and return the next one which is UP
+        If all endpoints are currently DOWN, depending on the configuration
+        retry it until one is UP (or max retries exceeded)
+        """
+        def _select_endpoint_internal(refresh=False):
+            # check for UP state until exhausting all endpoints
+            seen, total = 0, len(self._endpoints.values())
+            while seen < total:
+                endpoint = next(self._endpoint_schedule)
+                if refresh:
+                    self._validate(endpoint)
+                if endpoint.state == EndpointState.UP:
+                    return endpoint
+                seen += 1
+
+        @utils.retry_upon_none_result(self.nsxlib_config.max_attempts)
+        def _select_endpoint_internal_with_retry():
+            # redo endpoint selection with refreshing states
+            return _select_endpoint_internal(refresh=True)
+
+        # First attempt to get an UP endpoint
+        endpoint = _select_endpoint_internal()
+        if endpoint or not self.nsxlib_config.cluster_unavailable_retry:
+            return endpoint
+
+        # Retry the selection while refreshing the endpoints state
+        try:
+            return _select_endpoint_internal_with_retry()
+        except tenacity.RetryError:
+            # exhausted number of retries
+            return None
 
     def endpoint_for_connection(self, conn):
         # check all endpoint pools
@@ -584,10 +614,15 @@ class ClusteredAPI(object):
                 if not self._http_provider.is_connection_exception(e):
                     # only trap and retry connection errors
                     raise e
-                endpoint.set_state(EndpointState.DOWN)
+                if self._http_provider.is_conn_open_exception(e):
+                    # unable to establish new connection - endpoint is
+                    # inaccessible
+                    endpoint.set_state(EndpointState.DOWN)
+
                 LOG.debug("Connection to %s failed, checking additional "
-                          "endpoints" % url)
-                # retry until exhausting endpoints
+                          "connections and endpoints" % url)
+                # this might be a result of server closing connection
+                # retry until exhausting connections and endpoints
                 return self._proxy(proxy_for, uri, *args, **kwargs)
 
 
