@@ -124,6 +124,8 @@ class NsxLibLogicalSwitch(utils.NsxLibApiBase):
                mac_pool_id=None, description=None,
                trunk_vlan_range=None):
         operation = "Create logical switch"
+        if display_name:
+            display_name = utils.escape_display_name(display_name)
         # TODO(salv-orlando): Validate Replication mode and admin_state
         # NOTE: These checks might be moved to the API client library if one
         # that performs such checks in the client is available
@@ -189,6 +191,7 @@ class NsxLibLogicalSwitch(utils.NsxLibApiBase):
                description=None):
         body = {}
         if name:
+            name = utils.escape_display_name(name)
             body['display_name'] = name
         if admin_state is not None:
             if admin_state:
@@ -512,12 +515,29 @@ class NsxLibLogicalRouter(utils.NsxLibApiBase):
                                        'res': resource,
                                        'values': kwargs})
 
+    def _validate_nat_rule_action(self, action):
+        if not action:
+            return
+        if action in ['SNAT', 'DNAT', 'NO_NAT', 'REFLEXIVE']:
+            # legal values for all NSX versions
+            return
+        if (action not in ['NO_SNAT', 'NO_DNAT'] or (
+            self.nsxlib and not self.nsxlib.feature_supported(
+                nsx_constants.FEATURE_NO_DNAT_NO_SNAT))):
+            raise exceptions.InvalidInput(
+                operation="Create/Update NAT rule",
+                arg_val=action,
+                arg_name='action')
+
     def add_nat_rule(self, logical_router_id, action, translated_network,
                      source_net=None, dest_net=None,
                      enabled=True, rule_priority=None,
                      match_ports=None, match_protocol=None,
                      match_resource_type=None,
-                     bypass_firewall=True):
+                     bypass_firewall=True,
+                     tags=None,
+                     display_name=None):
+        self._validate_nat_rule_action(action)
         resource = 'logical-routers/%s/nat/rules' % logical_router_id
         body = {'action': action,
                 'enabled': enabled,
@@ -543,7 +563,10 @@ class NsxLibLogicalRouter(utils.NsxLibApiBase):
         elif not bypass_firewall:
             LOG.error("Ignoring bypass_firewall for router %s nat rule: "
                       "this feature is not supported.", logical_router_id)
-
+        if tags is not None:
+            body['tags'] = tags
+        if display_name:
+            body['display_name'] = display_name
         return self.client.create(resource, body)
 
     def add_static_route(self, logical_router_id, dest_cidr, nexthop):
@@ -593,6 +616,8 @@ class NsxLibLogicalRouter(utils.NsxLibApiBase):
         return self.client.list(resource)
 
     def update_nat_rule(self, logical_router_id, nat_rule_id, **kwargs):
+        if 'action' in kwargs:
+            self._validate_nat_rule_action(kwargs['action'])
         resource = 'logical-routers/%s/nat/rules/%s' % (
             logical_router_id, nat_rule_id)
         return self._update_resource(resource, kwargs, retry=True)
@@ -624,8 +649,22 @@ class NsxLibLogicalRouter(utils.NsxLibApiBase):
                     logical_router_id)
         return self.client.get(resource)
 
+    def get_debug_info(self, logical_router_id):
+        resource = ('logical-routers/%s/debug-info?format=text' %
+                    logical_router_id)
+        return self.client.get(resource)
+
+    def get_transportzone_id(self, logical_router_id):
+        res = self.get_debug_info(logical_router_id)
+        for item in res['componentInfo']:
+            if item['componentType'] == nsx_constants.ROUTER_TYPE_TIER0_DR:
+                if item['transportZoneId']:
+                    return item['transportZoneId'][0]
+        LOG.warning('OverlayTransportZone is not yet available on'
+                    ' %s.' % (logical_router_id))
+
     def create(self, display_name, tags, edge_cluster_uuid=None, tier_0=False,
-               description=None):
+               description=None, transport_zone_id=None, allocation_pool=None):
         # TODO(salv-orlando): If possible do not manage edge clusters
         # in the main plugin logic.
         router_type = (nsx_constants.ROUTER_TYPE_TIER0 if tier_0 else
@@ -637,6 +676,12 @@ class NsxLibLogicalRouter(utils.NsxLibApiBase):
             body['edge_cluster_id'] = edge_cluster_uuid
         if description:
             body['description'] = description
+        if transport_zone_id:
+            body['advanced_config'] = {
+                'transport_zone_id': transport_zone_id}
+        if allocation_pool:
+            body['allocation_profile'] = {
+                'allocation_pool': allocation_pool}
         return self.client.create(self.get_path(), body=body)
 
     def delete(self, lrouter_id, force=False):
@@ -646,7 +691,16 @@ class NsxLibLogicalRouter(utils.NsxLibApiBase):
         return self.client.delete(self.get_path(url))
 
     def update(self, lrouter_id, *args, **kwargs):
-        return self._update_with_retry(lrouter_id, kwargs)
+        body = {}
+        for arg in kwargs:
+            # special care for transport_zone_id
+            if arg == 'transport_zone_id':
+                body['advanced_config'] = {
+                    'transport_zone_id': kwargs['transport_zone_id']}
+            else:
+                body[arg] = kwargs[arg]
+
+        return self._update_with_retry(lrouter_id, body)
 
     def get_firewall_section_id(self, lrouter_id, router_body=None):
         """Return the id of the auto created firewall section of the router
@@ -682,6 +736,13 @@ class NsxLibEdgeCluster(utils.NsxLibApiBase):
     def resource_type(self):
         return 'EdgeCluster'
 
+    def get_transport_nodes(self, uuid):
+        ec = self.get(uuid)
+        members = []
+        for member in ec.get('members', []):
+            members.append(member.get('transport_node_id'))
+        return members
+
 
 class NsxLibTransportZone(utils.NsxLibApiBase):
 
@@ -711,6 +772,26 @@ class NsxLibTransportZone(utils.NsxLibApiBase):
         return tz.get('host_switch_mode', self.HOST_SWITCH_MODE_STANDARD)
 
 
+class NsxLibTransportNode(utils.NsxLibApiBase):
+
+    @property
+    def uri_segment(self):
+        return 'transport-nodes'
+
+    @property
+    def resource_type(self):
+        return 'TransportNode'
+
+    @property
+    def use_cache_for_get(self):
+        return True
+
+    def get_transport_zones(self, uuid):
+        tz = self.get(uuid)
+        return [ep.get('transport_zone_id') for ep in
+                tz.get('transport_zone_endpoints', [])]
+
+
 class NsxLibDhcpProfile(utils.NsxLibApiBase):
 
     @property
@@ -784,6 +865,12 @@ class NsxLibMetadataProxy(utils.NsxLibApiBase):
             body['edge_cluster_id'] = edge_cluster_id
         return self._update_with_retry(uuid, body)
 
+    def get_md_proxy_status(self, attachment_id, logical_switch_id):
+        """Return all matching logical port statuses"""
+        url_suffix = ('/%s/%s/status' %
+                      (attachment_id, logical_switch_id))
+        return self.client.get(self.get_path(url_suffix))
+
 
 class NsxLibBridgeCluster(utils.NsxLibApiBase):
 
