@@ -22,15 +22,31 @@ ACTION_ALLOW = 'ALLOW'
 ACTION_DENY = 'DROP'
 
 ANY_GROUP = 'ANY'
+ANY_SERVICE = 'ANY'
 
 CONDITION_KEY_TAG = 'Tag'
+CONDITION_KEY_NAME = 'Name'
 CONDITION_MEMBER_VM = 'VirtualMachine'
 CONDITION_MEMBER_PORT = 'LogicalPort'
 CONDITION_OP_EQUALS = 'EQUALS'
 CONDITION_OP_CONTAINS = 'CONTAINS'
 CONDITION_OP_STARTS_WITH = 'STARTSWITH'
+CONDITION_OP_AND = 'AND'
+CONDITION_OP_OR = 'OR'
 
 DEFAULT_THUMBPRINT = 'abc'
+DEFAULT_DOMAIN = 'default'
 
 STATE_REALIZED = 'REALIZED'
 STATE_UNREALIZED = 'UNREALIZED'
+
+CATEGORY_EMERGENCY = 'Emergency'
+CATEGORY_INFRASTRUCTURE = 'Infrastructure'
+CATEGORY_ENVIRONMENT = 'Environment'
+CATEGORY_APPLICATION = 'Application'
+
+ACTIVE_STANDBY = 'ACTIVE_STANDBY'
+ACTIVE_ACTIVE = 'ACTIVE_ACTIVE'
+
+PREEMPTIVE = 'PREEMPTIVE'
+NON_PREEMPTIVE = 'NON_PREEMPTIVE'
