@@ -131,7 +131,7 @@ class LogicalPort(utils.NsxLibApiBase):
 
     def _prepare_attachment(self, attachment_type, vif_uuid,
                             allocate_addresses, vif_type,
-                            parent_vif_id, traffic_tag, app_id):
+                            parent_vif_id, traffic_tag, app_id, tn_uuid):
         if attachment_type and vif_uuid:
             attachment = {'attachment_type': attachment_type,
                           'id': vif_uuid}
@@ -143,6 +143,9 @@ class LogicalPort(utils.NsxLibApiBase):
                     context['parent_vif_id'] = parent_vif_id
                     context['traffic_tag'] = traffic_tag
                     context['app_id'] = app_id
+                elif tn_uuid:
+                    context['transport_node_uuid'] = tn_uuid
+                    context['app_id'] = app_id
                 attachment['context'] = context
             return attachment
         elif attachment_type is None and vif_uuid is None:
@@ -156,7 +159,7 @@ class LogicalPort(utils.NsxLibApiBase):
                parent_vif_id=None, traffic_tag=None,
                switch_profile_ids=None, vif_type=None, app_id=None,
                allocate_addresses=nsx_constants.ALLOCATE_ADDRESS_NONE,
-               description=None):
+               description=None, tn_uuid=None):
         tags = tags or []
         body = {'logical_switch_id': lswitch_id}
         # NOTE(arosen): If parent_vif_id is specified we need to use
@@ -164,7 +167,7 @@ class LogicalPort(utils.NsxLibApiBase):
         attachment = self._prepare_attachment(attachment_type, vif_uuid,
                                               allocate_addresses, vif_type,
                                               parent_vif_id, traffic_tag,
-                                              app_id)
+                                              app_id, tn_uuid)
         body.update(self._build_body_attrs(
             display_name=name,
             admin_state=admin_state, tags=tags,
@@ -185,11 +188,11 @@ class LogicalPort(utils.NsxLibApiBase):
                parent_vif_id=None, traffic_tag=None,
                vif_type=None, app_id=None,
                allocate_addresses=nsx_constants.ALLOCATE_ADDRESS_NONE,
-               description=None):
+               description=None, tn_uuid=None):
         attachment = self._prepare_attachment(attachment_type, vif_uuid,
                                               allocate_addresses, vif_type,
                                               parent_vif_id, traffic_tag,
-                                              app_id)
+                                              app_id, tn_uuid)
         lport = {}
         if tags_update is not None:
             lport['tags_update'] = tags_update
@@ -343,6 +346,21 @@ class LogicalRouterPort(utils.NsxLibApiBase):
             manager=self.client.nsx_api_managers,
             operation="get router link port")
 
+    def get_tier0_uplink_port(self, logical_router_id):
+        logical_router_ports = self.get_by_router_id(logical_router_id)
+        for port in logical_router_ports:
+            if port['resource_type'] == nsx_constants.LROUTERPORT_UPLINK:
+                return port
+
+    def get_tier0_uplink_ips(self, logical_router_id):
+        port = self.get_tier0_uplink_port(logical_router_id)
+        ips = []
+        if port:
+            for subnet in port.get('subnets', []):
+                for ip_address in subnet.get('ip_addresses'):
+                    ips.append(ip_address)
+        return ips
+
 
 class MetaDataProxy(core_resources.NsxLibMetadataProxy):
     # TODO(asarfaty): keeping this for backwards compatibility.
@@ -405,6 +423,7 @@ class LogicalDhcpServer(utils.NsxLibApiBase):
             'domain-search': 119,
             'classless-static-route': 121,
             'tftp-server-address': 150,
+            'server-ip-address': 150,
             'etherboot': 175,
             'config-file': 209,
             'path-prefix': 210,
@@ -550,6 +569,12 @@ class IpPool(utils.NsxLibApiBase):
 
         return self.client.create(self.get_path(), body=body)
 
+    def delete(self, pool_id, force=False):
+        url = pool_id
+        if force:
+            url += '?force=%s' % force
+        return self.client.delete(self.get_path(url))
+
     def _update_param_in_pool(self, args_dict, key, pool_data):
         # update the arg only if it exists in the args dictionary
         if key in args_dict:
@@ -672,3 +697,32 @@ class NodeHttpServiceProperties(utils.NsxLibApiBase):
         """Not supported"""
         msg = _("Find is not supported for %s") % self.uri_segment
         raise exceptions.ManagerError(details=msg)
+
+
+class NsxlibClusterNodesConfig(utils.NsxLibApiBase):
+    @property
+    def uri_segment(self):
+        return 'cluster/nodes'
+
+    @property
+    def resource_type(self):
+        return 'ClusterNodeConfig'
+
+    def delete(self, uuid):
+        """Not supported"""
+        msg = _("Delete is not supported for %s") % self.uri_segment
+        raise exceptions.ManagerError(details=msg)
+
+    def get_managers_ips(self):
+        manager_ips = []
+        nodes_config = self.client.get(self.get_path())['results']
+        for node in nodes_config:
+            if 'manager_role' in node:
+                manager_conf = node['manager_role']
+                if 'api_listen_addr' in manager_conf:
+                    list_addr = manager_conf['mgmt_cluster_listen_addr']
+                    ip = list_addr['ip_address']
+                    if ip != '127.0.0.1':
+                        manager_ips.append(list_addr['ip_address'])
+
+        return manager_ips
