@@ -66,7 +66,7 @@ class RouterLib(object):
                     err_msg = _("%(act_num)s edge members found in "
                                 "edge_cluster %(cluster_id)s, however we "
                                 "require at least %(exp_num)s edge nodes "
-                                "in edge cluster for use.") % {
+                                "in edge cluster for use") % {
                         'act_num': len(member_index_list),
                         'exp_num': MIN_EDGE_NODE_NUM,
                         'cluster_id': edge_cluster_uuid}
@@ -96,7 +96,9 @@ class RouterLib(object):
             logical_port_id=linked_logical_port_id,
             address_groups=None)
 
-    def remove_router_link_port(self, tier1_uuid, tier0_uuid):
+    def remove_router_link_port(self, tier1_uuid, tier0_uuid=None):
+        # Note(asarfaty): tier0_uuid is not used by this method and can
+        # be removed.
         try:
             tier1_link_port = (
                 self._router_port_client.get_tier1_link_port(tier1_uuid))
@@ -153,18 +155,24 @@ class RouterLib(object):
             skip_not_found=True, strict_mode=False)
 
     def add_gw_snat_rule(self, logical_router_id, gw_ip, source_net=None,
-                         bypass_firewall=True):
+                         bypass_firewall=True, tags=None, display_name=None):
         return self.nsxlib.logical_router.add_nat_rule(
             logical_router_id, action="SNAT",
             translated_network=gw_ip,
             source_net=source_net,
             rule_priority=GW_NAT_PRI,
-            bypass_firewall=bypass_firewall)
+            bypass_firewall=bypass_firewall,
+            tags=tags,
+            display_name=display_name)
 
     def update_router_edge_cluster(self, nsx_router_id, edge_cluster_uuid):
         return self._router_client.update(nsx_router_id,
                                           edge_cluster_id=edge_cluster_uuid)
 
+    def update_router_transport_zone(self, nsx_router_id, transport_zone_id):
+        return self._router_client.update(nsx_router_id,
+                                          transport_zone_id=transport_zone_id)
+
     def create_logical_router_intf_port_by_ls_id(self, logical_router_id,
                                                  display_name,
                                                  tags,
@@ -194,20 +202,25 @@ class RouterLib(object):
                 relay_service_uuid=relay_service_uuid)
 
     def add_fip_nat_rules(self, logical_router_id, ext_ip, int_ip,
-                          match_ports=None, bypass_firewall=True):
+                          match_ports=None, bypass_firewall=True,
+                          tags=None, display_name=None):
         self.nsxlib.logical_router.add_nat_rule(
             logical_router_id, action="SNAT",
             translated_network=ext_ip,
             source_net=int_ip,
             rule_priority=FIP_NAT_PRI,
-            bypass_firewall=bypass_firewall)
+            bypass_firewall=bypass_firewall,
+            tags=tags,
+            display_name=display_name)
         self.nsxlib.logical_router.add_nat_rule(
             logical_router_id, action="DNAT",
             translated_network=int_ip,
             dest_net=ext_ip,
             rule_priority=FIP_NAT_PRI,
             match_ports=match_ports,
-            bypass_firewall=bypass_firewall)
+            bypass_firewall=bypass_firewall,
+            tags=tags,
+            display_name=display_name)
 
     def delete_fip_nat_rules_by_internal_ip(self, logical_router_id, int_ip):
         self.nsxlib.logical_router.delete_nat_rule_by_values(
@@ -241,3 +254,42 @@ class RouterLib(object):
         return self.nsxlib.logical_router.delete_static_route_by_values(
             nsx_router_id, dest_cidr=route['destination'],
             nexthop=route['nexthop'])
+
+    def get_tier0_router_tz(self, tier0_uuid):
+        lrouter = self._router_client.get(tier0_uuid)
+        edge_cluster_uuid = lrouter.get('edge_cluster_id')
+        if not edge_cluster_uuid:
+            return []
+        tier0_transport_nodes = self.nsxlib.edge_cluster.get_transport_nodes(
+            edge_cluster_uuid)
+        tier0_tzs = []
+        for tn_uuid in tier0_transport_nodes:
+            tier0_tzs.extend(self.nsxlib.transport_node.get_transport_zones(
+                tn_uuid))
+        return tier0_tzs
+
+    def get_tier0_router_overlay_tz(self, tier0_uuid):
+        tz_uuids = self.get_tier0_router_tz(tier0_uuid)
+        for tz_uuid in tz_uuids:
+            # look for the overlay tz
+            backend_type = self.nsxlib.transport_zone.get_transport_type(
+                tz_uuid)
+            if (backend_type ==
+                self.nsxlib.transport_zone.TRANSPORT_TYPE_OVERLAY):
+                return tz_uuid
+
+    def get_connected_t0_transit_net(self, tier1_uuid):
+        """Return the IP of the tier1->tier0 link port
+
+        return None if the router is not connected to a tier0 router
+        """
+        try:
+            tier1_link_port = (
+                self._router_port_client.get_tier1_link_port(tier1_uuid))
+        except exceptions.ResourceNotFound:
+            # No GW
+            return
+        for subnet in tier1_link_port.get('subnets', []):
+            for ip_address in subnet.get('ip_addresses'):
+                # Expecting only 1 ip here. Return it.
+                return ip_address
