@@ -18,10 +18,12 @@
 NSX-V3 Plugin security & Distributed Firewall integration module
 """
 
+from distutils import version
+
 from oslo_log import log
+from oslo_log import versionutils
 from oslo_utils import excutils
 
-from vmware_nsxlib.v3 import constants
 from vmware_nsxlib.v3 import exceptions
 from vmware_nsxlib.v3 import nsx_constants as consts
 from vmware_nsxlib.v3 import utils
@@ -30,7 +32,6 @@ from vmware_nsxlib.v3 import utils
 LOG = log.getLogger(__name__)
 
 PORT_SG_SCOPE = 'os-security-group'
-MAX_NSGROUPS_CRITERIA_TAGS = 10
 
 
 class NsxLibNsGroup(utils.NsxLibApiBase):
@@ -49,9 +50,9 @@ class NsxLibNsGroup(utils.NsxLibApiBase):
     def resource_type(self):
         return 'NSGroup'
 
-    def update_on_backend(self, context, security_group,
-                          nsgroup_id, section_id,
-                          log_sg_allowed_traffic):
+    def update_nsgroup_and_section(self, security_group,
+                                   nsgroup_id, section_id,
+                                   log_sg_allowed_traffic):
         name = self.get_name(security_group)
         description = security_group['description']
         logging = (log_sg_allowed_traffic or
@@ -62,15 +63,28 @@ class NsxLibNsGroup(utils.NsxLibApiBase):
         self.firewall_section.update(section_id, name, description,
                                      rules=rules)
 
+    def update_on_backend(self, context, security_group,
+                          nsgroup_id, section_id,
+                          log_sg_allowed_traffic):
+        # This api is deprecated because of the irrelevant context arg
+        versionutils.report_deprecated_feature(
+            LOG,
+            'security.NsxLibNsGroup.update_on_backend is deprecated. '
+            'Please use security.NsxLibNsGroup.update_nsgroup_and_section '
+            'instead.')
+        return self.update_nsgroup_and_section(security_group, nsgroup_id,
+                                               section_id,
+                                               log_sg_allowed_traffic)
+
     def get_name(self, security_group):
         # NOTE(roeyc): We add the security-group id to the NSGroup name,
         # for usability purposes.
         return '%(name)s - %(id)s' % security_group
 
     def get_lport_tags(self, secgroups):
-        if len(secgroups) > MAX_NSGROUPS_CRITERIA_TAGS:
+        if len(secgroups) > utils.MAX_NSGROUPS_CRITERIA_TAGS:
             raise exceptions.NumberOfNsgroupCriteriaTagsReached(
-                max_num=MAX_NSGROUPS_CRITERIA_TAGS)
+                max_num=utils.MAX_NSGROUPS_CRITERIA_TAGS)
         tags = []
         for sg in secgroups:
             tags = utils.add_v3_tag(tags, PORT_SG_SCOPE, sg)
@@ -79,9 +93,11 @@ class NsxLibNsGroup(utils.NsxLibApiBase):
             tags = [{'scope': PORT_SG_SCOPE, 'tag': None}]
         return tags
 
-    def update_lport(self, context, lport_id, original, updated):
-        added = set(updated) - set(original)
-        removed = set(original) - set(updated)
+    def update_lport_nsgroups(self, lport_id, original_nsgroups,
+                              updated_nsgroups):
+        """Update the NSgroups that the logical ports belongs to"""
+        added = set(updated_nsgroups) - set(original_nsgroups)
+        removed = set(original_nsgroups) - set(updated_nsgroups)
         for nsgroup_id in added:
             try:
                 self.add_members(
@@ -103,6 +119,14 @@ class NsxLibNsGroup(utils.NsxLibApiBase):
             self.remove_member(
                 nsgroup_id, consts.TARGET_TYPE_LOGICAL_PORT, lport_id)
 
+    def update_lport(self, context, lport_id, original, updated):
+        # This api is deprecated because of the irrelevant context arg
+        versionutils.report_deprecated_feature(
+            LOG,
+            'security.NsxLibNsGroup.update_lport is deprecated. '
+            'Please use security.NsxLibNsGroup.update_lport_nsgroups instead.')
+        return self.update_lport_nsgroups(lport_id, original, updated)
+
     def get_nsservice(self, resource_type, **properties):
         service = {'resource_type': resource_type}
         service.update(properties)
@@ -252,8 +276,8 @@ class NsxLibFirewallSection(utils.NsxLibApiBase):
         self._create_with_retry(resource, body)
 
     def remove_member_from_fw_exclude_list(self, target_id, target_type):
-        resource = ('firewall/excludelist?action=remove_member&object_id='
-                    + target_id)
+        resource = ('firewall/excludelist?action=remove_member&object_id=' +
+                    target_id)
         self._create_with_retry(resource)
 
     def get_excludelist(self):
@@ -265,34 +289,17 @@ class NsxLibFirewallSection(utils.NsxLibApiBase):
             else consts.OUT
         )
 
-    def _get_l4_protocol_name(self, protocol_number):
-        if protocol_number is None:
-            return
-        protocol_number = constants.IP_PROTOCOL_MAP.get(protocol_number,
-                                                        protocol_number)
-        protocol_number = int(protocol_number)
-        if protocol_number == 6:
-            return consts.TCP
-        elif protocol_number == 17:
-            return consts.UDP
-        elif protocol_number == 1:
-            return consts.ICMPV4
-        else:
-            return protocol_number
-
     def get_nsservice(self, resource_type, **properties):
         service = {'resource_type': resource_type}
         service.update(properties)
         return {'service': service}
 
     def _decide_service(self, sg_rule):
-        l4_protocol = self._get_l4_protocol_name(sg_rule['protocol'])
-        direction = self._get_direction(sg_rule)
+        l4_protocol = utils.get_l4_protocol_name(sg_rule['protocol'])
 
         if l4_protocol in [consts.TCP, consts.UDP]:
             # If port_range_min is not specified then we assume all ports are
             # matched, relying on neutron to perform validation.
-            source_ports = []
             if sg_rule['port_range_min'] is None:
                 destination_ports = []
             elif sg_rule['port_range_min'] != sg_rule['port_range_max']:
@@ -302,20 +309,25 @@ class NsxLibFirewallSection(utils.NsxLibApiBase):
             else:
                 destination_ports = ['%(port_range_min)s' % sg_rule]
 
-            if direction == consts.OUT:
-                source_ports, destination_ports = destination_ports, []
-
             return self.get_nsservice(
                 consts.L4_PORT_SET_NSSERVICE,
                 l4_protocol=l4_protocol,
-                source_ports=source_ports,
+                source_ports=[],
                 destination_ports=destination_ports)
         elif l4_protocol == consts.ICMPV4:
+            # Validate the icmp type & code
+            icmp_type = sg_rule['port_range_min']
+            icmp_code = sg_rule['port_range_max']
+            icmp_strict = self.nsxlib.feature_supported(
+                consts.FEATURE_ICMP_STRICT)
+            utils.validate_icmp_params(icmp_type, icmp_code, icmp_version=4,
+                                       strict=icmp_strict)
+
             return self.get_nsservice(
                 consts.ICMP_TYPE_NSSERVICE,
                 protocol=l4_protocol,
-                icmp_type=sg_rule['port_range_min'],
-                icmp_code=sg_rule['port_range_max'])
+                icmp_type=icmp_type,
+                icmp_code=icmp_code)
         elif l4_protocol is not None:
             return self.get_nsservice(
                 consts.IP_PROTOCOL_NSSERVICE,
@@ -404,8 +416,8 @@ class NsxLibFirewallSection(utils.NsxLibApiBase):
         return self.client.list(self.get_path()).get('results', [])
 
     def delete(self, section_id):
-        resource = '%s?cascade=true' % self.get_path(section_id)
-        return self.client.delete(resource)
+        resource = '%s?cascade=true' % section_id
+        return self._delete_with_retry(resource)
 
     def get_nsgroup_reference(self, nsgroup_id):
         return {'target_id': nsgroup_id,
@@ -457,14 +469,30 @@ class NsxLibFirewallSection(utils.NsxLibApiBase):
         return rule_dict
 
     def add_rule(self, rule, section_id, operation=consts.FW_INSERT_BOTTOM):
-        resource = '%s/rules' % self.get_path(section_id)
-        params = '?operation=%s' % operation
-        return self._create_with_retry(resource + params, rule)
+        @utils.retry_upon_exception(exceptions.StaleRevision,
+                                    max_attempts=self.client.max_attempts)
+        def do_add_rule():
+            resource = '%s/rules' % self.get_path(section_id)
+            params = '?operation=%s' % operation
+            if (version.LooseVersion(self.nsxlib.get_version()) >=
+                version.LooseVersion(consts.NSX_VERSION_2_4_0)):
+                rule['_revision'] = self.get(section_id)['_revision']
+            return self._create_with_retry(resource + params, rule)
+        return do_add_rule()
 
     def add_rules(self, rules, section_id, operation=consts.FW_INSERT_BOTTOM):
-        resource = '%s/rules' % self.get_path(section_id)
-        params = '?action=create_multiple&operation=%s' % operation
-        return self._create_with_retry(resource + params, {'rules': rules})
+        @utils.retry_upon_exception(exceptions.StaleRevision,
+                                    max_attempts=self.client.max_attempts)
+        def do_add_rules():
+            resource = '%s/rules' % self.get_path(section_id)
+            params = '?action=create_multiple&operation=%s' % operation
+            if (version.LooseVersion(self.nsxlib.get_version()) >=
+                version.LooseVersion(consts.NSX_VERSION_2_4_0)):
+                rev_id = self.get(section_id)['_revision']
+                for rule in rules:
+                    rule['_revision'] = rev_id
+            return self._create_with_retry(resource + params, {'rules': rules})
+        return do_add_rules()
 
     def delete_rule(self, section_id, rule_id):
         resource = '%s/rules/%s' % (section_id, rule_id)
@@ -516,9 +544,9 @@ class NsxLibFirewallSection(utils.NsxLibApiBase):
                                   [service] if service else None,
                                   action, logged)
 
-    def create_rules(self, context, section_id, nsgroup_id,
-                     logging_enabled, action, security_group_rules,
-                     ruleid_2_remote_nsgroup_map):
+    def create_section_rules(self, section_id, nsgroup_id,
+                             logging_enabled, action, security_group_rules,
+                             ruleid_2_remote_nsgroup_map):
         # 1. translate rules
         # 2. insert in section
         # 3. return the rules
@@ -530,9 +558,22 @@ class NsxLibFirewallSection(utils.NsxLibApiBase):
                 logging_enabled, action)
 
             firewall_rules.append(fw_rule)
-
         return self.add_rules(firewall_rules, section_id)
 
+    def create_rules(self, context, section_id, nsgroup_id,
+                     logging_enabled, action, security_group_rules,
+                     ruleid_2_remote_nsgroup_map):
+        # This api is deprecated because of the irrelevant context arg
+        versionutils.report_deprecated_feature(
+            LOG,
+            'security.NsxLibFirewallSection.create_rules is deprecated. '
+            'Please use security.NsxLibFirewallSection.create_section_rules '
+            'instead.')
+        return self.create_section_rules(
+            section_id, nsgroup_id,
+            logging_enabled, action, security_group_rules,
+            ruleid_2_remote_nsgroup_map)
+
     def set_rule_logging(self, section_id, logging):
         rules = self._process_rules_logging_for_update(
             section_id, logging)
@@ -549,14 +590,17 @@ class NsxLibFirewallSection(utils.NsxLibApiBase):
 
     def init_default(self, name, description, nested_groups,
                      log_sg_blocked_traffic):
+        LOG.info("Initializing the default section named %s", name)
         fw_sections = self.list()
         for section in reversed(fw_sections):
             if section['display_name'] == name:
+                LOG.info("Found existing default section %s", section['id'])
                 break
         else:
             tags = self.build_v3_api_version_tag()
             section = self.create_empty(
                 name, description, nested_groups, tags)
+            LOG.info("Creating a new default section %s", section['id'])
 
         block_rule = self.get_rule_dict(
             'Block All', action=consts.FW_ACTION_DROP,
