@@ -24,7 +24,9 @@ import tenacity
 from tenacity import _utils as tenacity_utils
 
 from vmware_nsxlib._i18n import _
+from vmware_nsxlib.v3 import constants
 from vmware_nsxlib.v3 import exceptions as nsxlib_exceptions
+from vmware_nsxlib.v3 import nsx_constants
 
 LOG = log.getLogger(__name__)
 
@@ -36,6 +38,7 @@ TagLimits = collections.namedtuple('TagLimits',
 MAX_RESOURCE_TYPE_LEN = 20
 MAX_TAG_LEN = 40
 MAX_TAGS = 15
+MAX_NSGROUPS_CRITERIA_TAGS = 10
 
 DEFAULT_MAX_ATTEMPTS = 10
 DEFAULT_CACHE_AGE_SEC = 600
@@ -74,10 +77,16 @@ def _update_max_tags(max_tags):
     MAX_TAGS = max_tags
 
 
+def _update_max_nsgroups_criteria_tags(max_tags):
+    global MAX_NSGROUPS_CRITERIA_TAGS
+    MAX_NSGROUPS_CRITERIA_TAGS = max(10, max_tags - 5)
+
+
 def update_tag_limits(limits):
     _update_resource_length(limits.scope_length)
     _update_tag_length(limits.tag_length)
     _update_max_tags(limits.max_tags)
+    _update_max_nsgroups_criteria_tags(limits.max_tags)
 
 
 def _validate_resource_type_length(resource_type):
@@ -175,6 +184,15 @@ def retry_random_upon_exception(exc, delay=0.5, max_delay=5,
                           before=_log_before_retry, after=_log_after_retry)
 
 
+def retry_upon_none_result(max_attempts, delay=0.5, max_delay=2):
+    return tenacity.retry(reraise=True,
+                          retry=tenacity.retry_if_result(lambda x: x is None),
+                          wait=tenacity.wait_exponential(
+                              multiplier=delay, max=max_delay),
+                          stop=tenacity.stop_after_attempt(max_attempts),
+                          before=_log_before_retry, after=_log_after_retry)
+
+
 def list_match(list1, list2):
     # Check if list1 and list2 have identical elements, but relaxed on
     # dict elements where list1's dict element can be a subset of list2's
@@ -239,6 +257,12 @@ def escape_tag_data(data):
     return data.replace('/', '\\/').replace('-', '\\-')
 
 
+def escape_display_name(display_name):
+    # Illegal characters for the display names are  ;|=,~@
+    rx = re.compile('([;|=,~@])')
+    return rx.sub('.', display_name)
+
+
 class NsxLibCache(object):
     def __init__(self, timeout):
         self.timeout = timeout
@@ -441,12 +465,10 @@ class NsxLibApiBase(object):
         if len(matched_results) == 0:
             err_msg = (_("Could not find %(resource)s %(name)s") %
                        {'name': name_or_id, 'resource': resource})
-            # TODO(aaron): improve exception handling...
             raise nsxlib_exceptions.ManagerError(details=err_msg)
         elif len(matched_results) > 1:
             err_msg = (_("Found multiple %(resource)s named %(name)s") %
                        {'name': name_or_id, 'resource': resource})
-            # TODO(aaron): improve exception handling...
             raise nsxlib_exceptions.ManagerError(details=err_msg)
 
         return matched_results[0].get('id')
@@ -508,3 +530,57 @@ class NsxLibApiBase(object):
                  'tag': project_name[:MAX_TAG_LEN]},
                 {'scope': 'os-api-version',
                  'tag': self.nsxlib_config.plugin_ver}]
+
+
+# Some utilities for services translations & validations
+# both for the nsx manager & policy manager
+def validate_icmp_params(icmp_type, icmp_code, icmp_version=4, strict=False):
+    if icmp_version != 4:
+        # ICMPv6 is currently not supported
+        return
+    if icmp_type:
+        if (strict and icmp_type not in
+                constants.IPV4_ICMP_STRICT_TYPES):
+            raise nsxlib_exceptions.InvalidInput(
+                operation='create_rule',
+                arg_val=icmp_type,
+                arg_name='icmp_type')
+        if icmp_type not in constants.IPV4_ICMP_TYPES:
+            raise nsxlib_exceptions.InvalidInput(
+                operation='create_rule',
+                arg_val=icmp_type,
+                arg_name='icmp_type')
+        if (icmp_code and strict and icmp_code not in
+                constants.IPV4_ICMP_STRICT_TYPES[icmp_type]):
+            raise nsxlib_exceptions.InvalidInput(
+                operation='create_rule',
+                arg_val=icmp_code,
+                arg_name='icmp_code for this icmp_type')
+        if (icmp_code and icmp_code not in
+                constants.IPV4_ICMP_TYPES[icmp_type]):
+            raise nsxlib_exceptions.InvalidInput(
+                operation='create_rule',
+                arg_val=icmp_code,
+                arg_name='icmp_code for this icmp_type')
+
+
+def get_l4_protocol_name(protocol_number):
+    if protocol_number is None:
+        return
+    protocol_number = constants.IP_PROTOCOL_MAP.get(protocol_number,
+                                                    protocol_number)
+    try:
+        protocol_number = int(protocol_number)
+    except ValueError:
+        raise nsxlib_exceptions.InvalidInput(
+            operation='create_rule',
+            arg_val=protocol_number,
+            arg_name='protocol')
+    if protocol_number == 6:
+        return nsx_constants.TCP
+    elif protocol_number == 17:
+        return nsx_constants.UDP
+    elif protocol_number == 1:
+        return nsx_constants.ICMPV4
+    else:
+        return protocol_number
